define('oxguard/mail/register_compose', [
    'io.ox/core/extensions',
    'gettext!oxguard/mail/register_compose',
    'io.ox/backbone/mini-views/dropdown',
    'oxguard/mail/oxguard_mail_compose_core',
    'oxguard/util',
    'oxguard/mail/options_new',
    'css!oxguard/style.css'
], function (ext, gt, Dropdown, core, util, options) {
    'use strict';

    var newpoint = ext.point('io.ox/mail/compose/fields');

    newpoint.extend({
        id: 'locksend',
        index: 'last',
        draw: function (baton) {
            // Create Security Link
            try {
                var view = baton.view.$el;
                if (view === undefined) return;//View not yet loaded
                if (view.find('#OxGuardOn').length > 0) return;//Already created
                options.createOptions(baton);
            } catch (e) {
                console.log(e.toString());
            }
        }
    });

    ext.point('io.ox/mail/compose/createtoken').extend({
        id: 'guardToken',
        action: function (baton) {
            if (baton.model.get('PGPGuard') === true) {
                var email = baton.event.attrs.model.get('token').value;
                var target = $(baton.event.relatedTarget);
                require(['oxguard/mail/keymanager'], function (keyman) {
                    var result = keyman.checkRecips (email, baton);
                    if (result !== null) {
                        core.drawKeyIcons(result, target);
                    }
                });
            }
        }
    });

    ext.point('io.ox/mail/compose/composetoolbar').extend({
        id: 'security',
        index: 150,
        draw: function (baton) {
            if (window.oxguarddata.settings) {
                if (window.oxguarddata.settings.oxguard) {
                    setQuit (baton);  // Set handles for quit function
                    var dropdown = new Dropdown({ model: baton.model, label: gt('Security'), caret: true });
                    dropdown
                        .header(gt('PGP'))
                        .option('PGPGuard', true, gt('Encrypt'))
                        .option('PGPSignature', true, gt('Sign'))
                        //#. Format of the email, HTML or Plaintext
                        .header(gt('Format'))
                        .option('PGPFormat', 'mime', gt('PGP Mime'))
                        .option('PGPFormat', 'inline', gt('PGP Inline'));
                    var div = this.find('.col-xs-4');
                    div.append(dropdown.render().$el.addClass('text-left').css('padding-left', '20px'));
                    this.append(div);
                }
            }
        }
    });

    // Check to see if Guard email if quiting.  If so, needs to save draft securely
    function setQuit (baton) {
        var app = baton.view.app;
        app.setQuit(function () {
            if (baton.model.get('PGPGuard') && app.view) {
                return guardQuit(baton, app);
            } else {
                if (app.view) return app.view.discard();
            }
        });
    }

    // Replace discard dialog so that drafts saved securely
    function guardQuit(baton, app) {
        var self = baton.view,
        def = $.Deferred();
        if (app.model.dirty()) {
            require(['io.ox/core/tk/dialogs'], function (dialogs) {
                //button texts may become quite large in some languages (e. g. french, see Bug 35581)
                //add some extra space
                //TODO maybe we could use a more dynamical approach
                new dialogs.ModalDialog({ width: 550, container: _.device('smartphone') ? self.$el.closest('.window-container-center') : $('#io-ox-core') })
                    .text(gt('Do you really want to discard your message?'))
                    //#. "Discard message" appears in combination with "Cancel" (this action)
                    //#. Translation should be distinguishable for the user
                    .addPrimaryButton('delete', gt.pgettext('dialog', 'Discard message'), 'delete', { tabIndex: 1 })
                    .addAlternativeButton('savedraft', gt('Save as draft'), 'savedraft', { tabIndex: 1 })
                    .addButton('cancel', gt('Cancel'), 'cancel', { tabIndex: 1 })
                    .show()
                    .done(function (action) {
                        if (action === 'delete') {
                            // clean before resolve, otherwise tinymce gets half-destroyed (ugly timing)
                            self.clean();
                            def.resolve();
                        } else if (action === 'savedraft') {
                            require (['oxguard/mail/sender_compose'], function (sender) {
                                sender.draftMail(baton, window.oxguarddata.passcode)
                                .always(function () {
                                    self.clean();
                                }).done(def.resolve).fail(def.reject);
                            });
                        } else {
                            def.reject();
                        }
                    });
            });
        } else {
            self.clean();
            def.resolve();
        }

        return def;
    }

    var sendpoint = ext.point('io.ox/mail/compose/buttons');
    sendpoint.extend({
        id: 'send',
        index: 100,
        draw: function (baton) {
            this.append($('<button type="button" class="btn btn-primary" data-action="send">')
                .on('click', function () {
                    if (baton.model.get('PGPGuard')) {
                        require (['oxguard/mail/sender_compose'], function (sender) {
                            sender.sendMail(baton, window.oxguarddata.passcode);
                        });
                    } else {
                        if (baton.model.get('PGPSignature')) { // If for PGP Signature
                            if (!util.isGuardConfigured()) {  // Check if user already set up
                                require (['oxguard/oxguard_core'], function (core) {
                                    core.createKeysWizard()
                                    .done(function () {
                                        sendSigned(baton);
                                    });
                                });
                                return;// Do not send until the master key is created
                            } else {
                                sendSigned(baton);
                            }

                        } else {
                            baton.view.send();
                        }
                    }
                })
                .text(gt('Send')));
        }
    });

    // Send plaintext signed email
    function sendSigned (baton) {
        if (baton.model.get('headers') === undefined || baton.model.get('headers')['X-OX-Guard-Auth'] === undefined) {// If not already authrized
            require(['oxguard/oxguard_core', 'oxguard/crypto/encr', 'io.ox/core/notifications'], function (core, encr, notify) {
                core.getPassword(gt('Please enter your Guard password to sign this email'))  //Get password
                .done(function (pass) {
                    core.auth(ox.user_id, pass)  // Verify against backend
                    .done(function (data) {
                        if (data.auth.length > 20) {  // If OK, add header and send
                            var headers = baton.model.get('headers');
                            if (headers === undefined) headers = {};
                            var newheaders = {
                                'X-OX-Guard-Auth': encr.cryptPass(pass),
                                'X-OX-Guard-Marker': true
                            };
                            var headers = _.extend(headers, newheaders);
                            baton.model.set('headers', headers);
                            baton.view.send();
                        } else {
                            // Otherwise report error
                            if (data.auth === 'Bad Password') {
                                notify.yell('error', gt('Bad Password'));
                            } else {
                                notify.yell('error', gt('Error') + ' ' + data.auth);
                            }
                        }
                    })
                    .fail(function () {
                        notify.yell('error', gt('Unable to authorize against backend'));
                    });
                });
            });
        } else {
            baton.model.get('headers')['X-OX-Guard-Marker'] = true;
            baton.view.send();
        }
    }

    ext.point('io.ox/mail/compose/buttons').replace({
        id: 'save',
        index: 200,
        draw: function (baton) {
            this.append($('<button type="button" class="btn btn-default" data-action="save">')
                .on('click', function () {
                    if (baton.view.isSaving === true) return false;
                    baton.view.isSaving = true;
                    if (baton.view.model.get('PGPGuard')) {
                        if (!util.isGuardConfigured()) {
                            require (['oxguard/oxguard_core'], function (core) {
                                core.createKeysWizard()
                                .done(function () {
                                    saveDraft(baton);
                                });
                            });
                            baton.view.isSaving = false;
                            return;// Do not send until the master key is created
                        }
                        saveDraft (baton);
                        return;
                    }
                    baton.view.saveDraft().done(function () {
                        baton.view.isSaving = false;
                    }).fail(function () {
                        baton.view.isSaving = false;
                    });
                })
                .text(gt('Save')));
        }
    });

    function saveDraft (baton) {
        require (['oxguard/mail/sender_compose'], function (sender) {
            baton.view.isSaving = true;
            sender.draftMail(baton, window.oxguarddata.passcode)
            .then(function () {
                baton.view.isSaving = false;
            });
        });
    }
});
